@@ -24,7 +24,7 @@
 #include <map>
 #include <stdexcept>
 
-#include "regex.h"
+#include "vectors.h"
 
 class ConfFileException : public std::exception {
 protected:
