@@ -33,27 +33,50 @@
 SDLAppDisplay display;
 
 SDLAppDisplay::SDLAppDisplay() {
-    clearColour = vec4f(0.0f,0.0f,0.0f,1.0f);
+    clearColour = vec4(0.0f,0.0f,0.0f,1.0f);
     enable_shaders=false;
     enable_alpha=false;
     vsync=false;
+    resizable=false;
+    frameless=false;
     multi_sample = 0;
+    width = 0;
+    height = 0;
+    desktop_width   = 0;
+    desktop_height  = 0;
+    windowed_width  = 0;
+    windowed_height = 0;
+#if SDL_VERSION_ATLEAST(1,3,0)
+    sdl_window = 0;
+    gl_context = 0;
+#else
+    surface = 0;
+#endif
+
 }
 
 SDLAppDisplay::~SDLAppDisplay() {
 }
 
-void SDLAppDisplay::setClearColour(vec3f colour) {
-    setClearColour(vec4f(colour, enable_alpha ? 0.0f : 1.0f));
+void SDLAppDisplay::setClearColour(vec3 colour) {
+    setClearColour(vec4(colour, enable_alpha ? 0.0f : 1.0f));
 }
 
-void SDLAppDisplay::setClearColour(vec4f colour) {
+void SDLAppDisplay::setClearColour(vec4 colour) {
     clearColour = colour;
 }
 
 int SDLAppDisplay::SDLFlags(bool fullscreen) {
     int flags = SDL_OPENGL | SDL_HWSURFACE | SDL_ANYFORMAT | SDL_DOUBLEBUF;
+    if (frameless) flags |= SDL_NOFRAME;
+    if (resizable && !fullscreen) flags |= SDL_RESIZABLE;
     if (fullscreen) flags |= SDL_FULLSCREEN;
+
+    //SDL 1.3 vsync
+#ifdef SDL_RENDERER_PRESENTVSYNC
+    if (!vsync) flags |= SDL_RENDERER_PRESENTVSYNC;
+#endif
+
     return flags;
 }
 
@@ -65,6 +88,10 @@ void SDLAppDisplay::enableShaders(bool enable) {
     enable_shaders = enable;
 }
 
+void SDLAppDisplay::enableResize(bool resizable) {
+    this->resizable = resizable;
+}
+
 void SDLAppDisplay::enableAlpha(bool enable) {
     enable_alpha = enable;
 }
@@ -92,23 +119,37 @@ bool SDLAppDisplay::multiSamplingEnabled() {
     return value==1;
 }
 
+void SDLAppDisplay::setVideoMode(int width, int height, bool fullscreen) {
+#if SDL_VERSION_ATLEAST(1,3,0)
 
-void SDLAppDisplay::init(std::string window_title, int width, int height, bool fullscreen) {
+    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
+    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
 
-    this->fullscreen = fullscreen;
+    Uint32 flags = SDL_WINDOW_OPENGL | SDL_WINDOW_SHOWN;
+    if(resizable && !fullscreen) flags |= SDL_WINDOW_RESIZABLE;
+    if(fullscreen) flags |= SDL_WINDOW_FULLSCREEN;
 
-    int flags = SDLFlags(fullscreen);
-    int depth = 32;
+    if(gl_context != 0) SDL_GL_DeleteContext(gl_context);
+    if(sdl_window != 0) SDL_DestroyWindow(sdl_window);
 
-    if(SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO) != 0) {
-        throw SDLInitException(SDL_GetError());
+    sdl_window = SDL_CreateWindow(
+	gSDLAppTitle.c_str(), SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
+        width, height, flags);
+
+
+    if (!sdl_window) {
+        std::string sdlerr(SDL_GetError());
+        throw SDLInitException(sdlerr);
     }
 
-    atexit(SDL_Quit);
+    gl_context = SDL_GL_CreateContext(sdl_window);
 
-    SDL_EnableUNICODE(1);
+    if(vsync) SDL_GL_SetSwapInterval(1);
+#else
+    int depth = 32;
+
+    int flags = SDLFlags(fullscreen);
 
-    //vsync
     if(vsync) SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);
     else SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 0);
 
@@ -153,20 +194,144 @@ void SDLAppDisplay::init(std::string window_title, int width, int height, bool f
             throw SDLInitException(sdlerr);
         }
     }
+#endif
+
+    setupExtensions();
+}
+
+void SDLAppDisplay::toggleFullscreen() {
+
+    int width  = this->width;
+    int height = this->height;
+
+    if(!fullscreen) {
+
+        //save windowed width and height
+        windowed_width  = width;
+        windowed_height = height;
+
+        int fullscreen_width  = desktop_width;
+        int fullscreen_height = desktop_height;
+            
+        float aspect_ratio = fullscreen_width / (float) fullscreen_height;
+        
+        // if the aspect ratio suggests the person is using multiple monitors
+        // find a supported resolution with a lower aspect ratio with the same
+        // fullscreen height
+
+#if SDL_VERSION_ATLEAST(1,3,0)
+        // TODO: do something with the 1.3 API here
+#else
+        if(aspect_ratio >= 2.5) {
+        
+            SDL_Rect** modes = SDL_ListModes(0, SDLFlags(true));
+        
+            if(modes != (SDL_Rect**)0 && modes != (SDL_Rect**)-1) {
+            
+                for (int i=0; modes[i]; i++) {
+                    if(modes[i]->h == fullscreen_height && (modes[i]->w/(float)modes[i]->h) < 2.5) {
+                        fullscreen_width = modes[i]->w;
+                        break;
+                    }
+                }
+            }
+        }
+#endif
+
+        width  = fullscreen_width;
+        height = fullscreen_height;
+        
+    } else {
+        //switch back to window dimensions, if known
+        if(windowed_width != 0) {
+            width  = windowed_width;
+            height = windowed_height;
+        }
+    }
+
+    fullscreen = !fullscreen;
+
+    setVideoMode(width, height, fullscreen);
+
+    int resized_width, resized_height;
+
+#if SDL_VERSION_ATLEAST(1,3,0)
+    SDL_GetWindowSize(sdl_window, &resized_width, &resized_height);
+#else
+    const SDL_VideoInfo* display_info = SDL_GetVideoInfo();
+
+    resized_width  = display_info->current_w;
+    resized_height = display_info->current_h;
+#endif
+
+    //set viewport to match what we ended up on
+    glViewport(0, 0, resized_width, resized_height);
+
+    this->width  = resized_width;
+    this->height = resized_height;
+}
+
+void SDLAppDisplay::resize(int width, int height) {
+
+    int resized_width, resized_height;
+
+#if SDL_VERSION_ATLEAST(1,3,0)
+    SDL_GetWindowSize(sdl_window, &resized_width, &resized_height);
+#else
+    setVideoMode(width, height, fullscreen);
+    
+    const SDL_VideoInfo* display_info = SDL_GetVideoInfo();
+
+    resized_width  = display_info->current_w;
+    resized_height = display_info->current_h;
+#endif
+
+    //set viewport to match what we ended up on
+    glViewport(0, 0, resized_width, resized_height);
+
+    this->width  = resized_width;
+    this->height = resized_height;
+}
+
+void SDLAppDisplay::init(std::string window_title, int width, int height, bool fullscreen) {
+
+    if(SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO) != 0) {
+        throw SDLInitException(SDL_GetError());
+    }
+
+    const SDL_VideoInfo* display_info = SDL_GetVideoInfo();
+
+    //save the desktop resolution
+    desktop_width  = display_info->current_w;
+    desktop_height = display_info->current_h;
+      
+    atexit(SDL_Quit);
+
+    SDL_EnableUNICODE(1);
+
+#if SDL_VERSION_ATLEAST(1,3,0)
+#else
+    SDL_WM_SetCaption(window_title.c_str(),0);
+#endif
+
+    setVideoMode(width, height, fullscreen);
 
     //get actual opengl viewport
     GLint viewport[4];
     glGetIntegerv( GL_VIEWPORT, viewport );
 
-    this->width  = viewport[2];
-    this->height = viewport[3];
-
-    setupExtensions();
-
-    SDL_WM_SetCaption(window_title.c_str(),0);
+    this->width      = viewport[2];
+    this->height     = viewport[3];
+    this->fullscreen = fullscreen;
 }
 
 void SDLAppDisplay::quit() {
+
+#if SDL_VERSION_ATLEAST(1,3,0)
+    if(gl_context != 0) SDL_GL_DeleteContext(gl_context);
+    if(sdl_window != 0) SDL_DestroyWindow(sdl_window);
+#endif
+
     texturemanager.purge();
     shadermanager.purge();
     fontmanager.purge();
@@ -174,7 +339,11 @@ void SDLAppDisplay::quit() {
 }
 
 void SDLAppDisplay::update() {
+#if SDL_VERSION_ATLEAST(1,3,0)
+    SDL_GL_SwapWindow(sdl_window);
+#else
     SDL_GL_SwapBuffers();
+#endif
 }
 
 void SDLAppDisplay::clear() {
@@ -216,92 +385,13 @@ void SDLAppDisplay::pop2D() {
         glPopMatrix();
 }
 
-vec4f SDLAppDisplay::currentColour() {
-    vec4f colour;
-   	glGetFloatv(GL_CURRENT_COLOR, colour);
-   	return colour;
+vec4 SDLAppDisplay::currentColour() {
+    vec4 colour;
+    glGetFloatv(GL_CURRENT_COLOR, glm::value_ptr(colour));
+    return colour;
 }
 
-void SDLAppDisplay::checkGLErrors() {
-//TODO
-}
-
-void SDLAppDisplay::fullScreenQuad(bool coord_flip) {
-
-    int y1 = coord_flip ? 0 : 1;
-    int y2 = coord_flip ? 1 : 0;
-
-    glBegin(GL_QUADS);
-        glTexCoord2i(0,y1);
-        glVertex2f(0, 0);
-
-        glTexCoord2i(1,y1);
-        glVertex2i(display.width, 0);
-
-        glTexCoord2i(1,y2);
-        glVertex2i(display.width, display.height);
-
-        glTexCoord2i(0,y2);
-        glVertex2i(0, display.height);
-    glEnd();
-}
-
-void SDLAppDisplay::renderToTexture(GLuint target, int width, int height, GLenum format) {
-    glEnable(GL_TEXTURE_2D);
-    glBindTexture(GL_TEXTURE_2D, target);
-    glCopyTexImage2D(GL_TEXTURE_2D, 0, format, 0, 0, width, height, 0);
-}
-
-GLuint SDLAppDisplay::emptyTexture(int width, int height, GLenum format) {
-    GLuint textureid;
-
-    glEnable(GL_TEXTURE_2D);
-
-    glGenTextures(1, &textureid);
-    glBindTexture(GL_TEXTURE_2D, textureid);
-
-    glTexImage2D(GL_TEXTURE_2D, 0, 4, width, height, 0,
-        GL_RGBA, GL_UNSIGNED_BYTE, 0);
-
-    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
-    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
-
-    return textureid;
-}
-
-GLuint SDLAppDisplay::createTexture(int width, int height, bool mipmaps, bool clamp, bool trilinear, GLenum format, unsigned int* data) {
-
-    GLuint textureid;
-
-    glGenTextures(1, &textureid);
-    glBindTexture(GL_TEXTURE_2D, textureid);
-
-    if(mipmaps) {
-        gluBuild2DMipmaps(GL_TEXTURE_2D, 4, width, height, format, GL_UNSIGNED_BYTE, data);
-    } else {
-    	glTexImage2D(GL_TEXTURE_2D, 0, 4, width, height, 0, format, GL_UNSIGNED_BYTE, data);
-    }
-
-    //GL_LINEAR_MIPMAP_LINEAR  - Trilinear
-    //GL_LINEAR_MIPMAP_NEAREST - Bilinear
-
-    if(trilinear) {
-        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
-    } else {
-        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
-    }
-
-    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST_MIPMAP_NEAREST);
-
-    if(clamp) {
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-    }
-
-    return textureid;
-}
-
-vec3f SDLAppDisplay::project(vec3f pos) {
+vec3 SDLAppDisplay::project(vec3 pos) {
     GLint viewport[4];
     GLdouble modelview[16];
     GLdouble projection[16];
@@ -315,10 +405,10 @@ vec3f SDLAppDisplay::project(vec3f pos) {
 
     winY = (float)viewport[3] - winY;
 
-    return vec3f((float) winX, (float) winY, (float) winZ);
+    return vec3((float) winX, (float) winY, (float) winZ);
 }
 
-vec3f SDLAppDisplay::unproject(vec2f pos) {
+vec3 SDLAppDisplay::unproject(vec2 pos) {
     GLint viewport[4];
     GLdouble modelview[16];
     GLdouble projection[16];
@@ -334,5 +424,5 @@ vec3f SDLAppDisplay::unproject(vec2f pos) {
     glReadPixels( int(winX), int(winY), 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winZ );
     gluUnProject( winX, winY, winZ, modelview, projection, viewport, &posX, &posY, &posZ);
 
-    return vec3f((float) posX, (float) posY, (float) posZ);
+    return vec3((float) posX, (float) posY, (float) posZ);
 }
