@@ -55,10 +55,11 @@ public:
 };
 
 class SDLAppDisplay {
-    SDL_Surface *surface;
 
     bool enable_shaders;
     bool enable_alpha;
+    bool resizable;
+    bool frameless;
     bool vsync;
 
     int  multi_sample;
@@ -67,13 +68,27 @@ class SDLAppDisplay {
     void setupExtensions();
 public:
     int width, height;
+    int desktop_width, desktop_height;
+    int windowed_width, windowed_height;
+
+#if SDL_VERSION_ATLEAST(1,3,0)
+    SDL_Window*   sdl_window;
+    SDL_GLContext gl_context;
+#else
+    SDL_Surface *surface;
+#endif
+
     bool fullscreen;
-    vec4f clearColour;
+    vec4 clearColour;
 
     SDLAppDisplay();
     ~SDLAppDisplay();
 
-    void   init(std::string window_title, int xres, int yres, bool fullscreen);
+    void   toggleFullscreen();
+    void   resize(int width, int height);
+
+    void   init(std::string window_title, int width, int height, bool fullscreen);
+    void   setVideoMode(int width, int height, bool fullscreen);
 
     bool   multiSamplingEnabled();
 
@@ -81,13 +96,17 @@ public:
 
     void   update();
     void   clear();
-    void   enableVsync(bool vsync);
-    void   setClearColour(vec3f colour);
-    void   setClearColour(vec4f colour);
+
+    void   setClearColour(vec3 colour);
+    void   setClearColour(vec4 colour);
+
 
     void   enableShaders(bool enable);
-
+    void   enableVsync(bool vsync);
     void   enableAlpha(bool enable);
+    void   enableResize(bool resizable);
+    void   enableFrameless(bool frameless);
+
     void   multiSample(int sample);
 
     void   mode3D(float fov, float znear, float zfar);
@@ -96,18 +115,10 @@ public:
     void   push2D();
     void   pop2D();
 
-    vec4f  currentColour();
+    vec4  currentColour();
 
-    void fullScreenQuad(bool coord_flip);
-
-    void   renderToTexture(GLuint target, int width, int height, GLenum format);
-    GLuint emptyTexture(int width, int height, GLenum format);
-    GLuint createTexture(int width, int height, bool mipmaps, bool clamp, bool trilinear, GLenum format, unsigned int* data);
-
-    vec3f project(vec3f pos);
-    vec3f unproject(vec2f pos);
-
-    void checkGLErrors();
+    vec3 project(vec3 pos);
+    vec3 unproject(vec2 pos);
 };
 
 extern SDLAppDisplay display;
