@@ -76,6 +76,9 @@ FXGlyph::FXGlyph(FXGlyphSet* set, unsigned int chr) {
     if(FT_Get_Glyph( ftface->glyph, &ftglyph ))
         throw FXFontException(ftface->family_name);
 
+    FT_Glyph_Metrics *metrics = &ftface->glyph->metrics;
+    height = glm::ceil(metrics->height / 64.0);
+
     FT_Glyph_To_Bitmap( &ftglyph, FT_RENDER_MODE_NORMAL, 0, 1 );
 
     glyph_bitmap = (FT_BitmapGlyph)ftglyph;
@@ -205,6 +208,9 @@ FXGlyphSet::FXGlyphSet(FT_Library freetype, const std::string& fontfile, int siz
     this->dpi      = dpi;
     this->ftface   = 0;
 
+    this->tab_width  = 4.0f;
+    this->max_height = 0;
+
     init();
 }
 
@@ -284,6 +290,8 @@ FXGlyph* FXGlyphSet::getGlyph(unsigned int chr) {
         }
     }
 
+    max_height = glm::max( glyph->getHeight(), max_height );
+
     //update the texture unless this is the precaching process
     if(!pre_caching) page->updateTexture();
 
@@ -297,7 +305,7 @@ float FXGlyphSet::getMaxWidth() const {
 }
 
 float FXGlyphSet::getMaxHeight() const {
-    return (ftface->bbox.yMax - ftface->bbox.yMin) / 64.0f;
+    return max_height;
 }
 
 float FXGlyphSet::getWidth(const std::string& text) {
@@ -317,6 +325,10 @@ float FXGlyphSet::getWidth(const std::string& text) {
     return width;
 }
 
+float FXGlyphSet::getAscender() const {
+    return getFTFace()->ascender / 64.0f;
+}
+
 void FXGlyphSet::drawToVBO(vec2& cursor, const std::string& text, const vec4& colour) {
     FTUnicodeStringItr<unsigned char> unicode_text((const unsigned char*)text.c_str());
 
@@ -341,11 +353,26 @@ void FXGlyphSet::draw(const std::string& text) {
 
     vec2 pos;
 
+    std::vector<FXGlyph*> glyphs;
+    glyphs.reserve(text.size());
+
+    // iterate over glyphs before drawing to avoid
+    // encountering a new glyph while inside the GL draw call
+
     while (*unicode_text) {
         chr = *unicode_text++;
 
+        if(chr == '\t') {
+             FXGlyph* glyph = getGlyph('M');
+             pos += glyph->getAdvance() * tab_width;
+             continue;
+        }
         FXGlyph* glyph = getGlyph(chr);
 
+        glyphs.push_back(glyph);
+    }
+
+    for(auto glyph : glyphs) {
         if(glyph->page->texture->textureid != textureid) {
             if(textureid != -1) glEnd();
             textureid = glyph->page->texture->textureid;
@@ -357,11 +384,11 @@ void FXGlyphSet::draw(const std::string& text) {
 
         pos += glyph->getAdvance();
     }
+
     if(textureid != -1) glEnd();
 }
 
 void FXGlyphSet::drawPages() {
-    vec2 corner = vec2(0.0f);
 
     glPushMatrix();
 
@@ -467,6 +494,10 @@ float FXFont::getWidth(const std::string& text) const {
     return glyphset->getWidth(text);
 }
 
+float FXFont::getAscender() const {
+    return glyphset->getAscender();
+}
+
 void FXFont::render(float x, float y, const std::string& text, const vec4& colour) const{
 
     if(fontmanager.use_vbo) {
@@ -507,7 +538,6 @@ void FXFont::draw(float x, float y, const std::string& text) const {
     }
 
     if(align_top) {
-        //fprintf(stderr, "font size = %d, max height = %.2f, ascender = %.2f, descender = %.2f\n", getFontSize(), getMaxHeight(), (glyphset->getFTFace()->ascender / 64.0f), (glyphset->getFTFace()->descender / 64.0f));
         y += getFontSize() + (glyphset->getFTFace()->descender / 64.0f);
     }
 
