@@ -76,9 +76,9 @@ class FXGlyphSet;
 
 class FXGlyphPage {
     GLubyte* texture_data;
-    bool needs_update;
-    int page_width;
-    int page_height;
+    bool     needs_update;
+    int      page_width;
+    int      page_height;
 public:
     TextureResource* texture;
 
@@ -99,6 +99,7 @@ class FXGlyph {
     vec2 dims;
     vec2 corner;
     vec2 advance;
+    int  height;
 public:
     vec2 vertex_positions[4];
     vec2 vertex_texcoords[4];
@@ -114,6 +115,8 @@ public:
     const vec2& getCorner() const { return corner; };
     const vec2& getDimensions() const { return dims; };
 
+    int getHeight() const { return height; };
+
     void setPage(FXGlyphPage* page, const vec4& texcoords);
 
     void drawToVBO(quadbuf& buffer, const vec2& offset, const vec4& colour) const;
@@ -124,8 +127,10 @@ class FXGlyphSet {
     FT_Library freetype;
     FT_Face ftface;
     std::string fontfile;
+    int max_height;
     int size;
     int dpi;
+    float tab_width;
 
     bool pre_caching;
 
@@ -147,7 +152,9 @@ public:
 
     float getWidth(const std::string& text);
 
-    float getMaxWidth() const;
+    float getAscender() const;
+
+    float getMaxWidth()  const;
     float getMaxHeight() const;
 
     int getSize() const { return size; };
@@ -186,6 +193,8 @@ public:
 
     float getWidth(const std::string& text) const;
 
+    float getAscender() const;
+
     void alignTop(bool top);
     void alignRight(bool right);
 
