@@ -26,13 +26,45 @@
 */
 
 #include "logger.h"
+#include <boost/assign/list_of.hpp>
+
+#include <stdio.h>
+#include <stdarg.h>
+#include <stdlib.h>
 
 std::map<int,std::string> log_levels = boost::assign::map_list_of
-    (LOG_LEVEL_WARN,     " WARN" )
-    (LOG_LEVEL_DEBUG,    "DEBUG" )
-    (LOG_LEVEL_INFO,     " INFO" )
-    (LOG_LEVEL_ERROR,    "ERROR" )
-    (LOG_LEVEL_INFINITY, "????"  );
+    (LOG_LEVEL_ERROR,    "  ERROR" )
+    (LOG_LEVEL_CONSOLE,  "CONSOLE" )
+    (LOG_LEVEL_INFO,     "   INFO" )
+    (LOG_LEVEL_SCRIPT,   " SCRIPT" )
+    (LOG_LEVEL_DEBUG,    "  DEBUG" )
+    (LOG_LEVEL_WARN,     "   WARN" )
+    (LOG_LEVEL_PEDANTIC, " PEDANT" );
+
+#define PARSE_AND_LOG(LOG_LEVEL) \
+    Logger* logger = Logger::getDefault(); \
+\
+if(!logger || logger->getLevel() > LOG_LEVEL) return; \
+\
+    char msgbuff[65536]; \
+    char* buffer = msgbuff; \
+\
+    va_list vl; \
+\
+    va_start(vl, str); \
+        int string_size = vsnprintf(buffer, sizeof(msgbuff), str, vl) + 1; \
+\
+        if(string_size > sizeof(msgbuff)) { \
+            buffer = new char[string_size]; \
+            string_size = vsnprintf(buffer, string_size, str, vl); \
+        } \
+    \
+    va_end(vl); \
+\
+\
+    logger->message( LOG_LEVEL, buffer ); \
+\
+    if(buffer != msgbuff) delete[] buffer;
 
 // LoggerMessage
 
@@ -42,6 +74,10 @@ LoggerMessage::LoggerMessage(int level, const std::string& message)
 
 // Logger
 
+Logger* Logger::getDefault() {
+    return default_logger;
+}
+
 Logger::Logger(int level, FILE* stream, int hist_capacity) {
     init(level, stream, hist_capacity);
 }
@@ -50,14 +86,19 @@ void Logger::init(int level, FILE* stream, int hist_capacity) {
     this->level         = level;
     this->stream        = stream;
     this->hist_capacity = hist_capacity;
+    this->message_count = 0;
 }
 
-    void Logger::message(int level, const std::string& message) {
+int Logger::getMessageCount() {
+    return message_count;
+}
 
-    if(this->level > level) return;
+void Logger::message(int level, const std::string& message) {
+
+    if(!level || this->level < level) return;
 
     if(stream != 0) {
-        fprintf(stderr, "%s: %s\n", log_levels[level].c_str(), message.c_str());
+        fprintf(stream, "%s: %s\n", log_levels[level].c_str(), message.c_str());
     }
 
     if(!hist_capacity) return;
@@ -67,66 +108,43 @@ void Logger::init(int level, FILE* stream, int hist_capacity) {
     }
 
     history.push_back(LoggerMessage(level, message));
+    message_count++;
+}
+
+const std::deque<LoggerMessage>& Logger::getHistory() const {
+    return history;
+}
+
+void Logger::setHistoryCapacity(int hist_capacity) {
+    this->hist_capacity = hist_capacity;
 }
 
 void warnLog(const char *str, ...) {
-
-    if(!logger || logger->getLevel() > LOG_LEVEL_WARN) return;
-
-    char msgbuff[65536];
-
-    va_list vl;
-
-    va_start(vl, str);
-        vsnprintf(msgbuff, 65536, str, vl);
-    va_end(vl);
-
-    logger->message( LOG_LEVEL_WARN, msgbuff );
+    PARSE_AND_LOG(LOG_LEVEL_WARN);
 }
 
 void debugLog(const char *str, ...) {
-
-    if(!logger || logger->getLevel() > LOG_LEVEL_DEBUG) return;
-
-    char msgbuff[65536];
-
-    va_list vl;
-
-    va_start(vl, str);
-        vsnprintf(msgbuff, 65536, str, vl);
-    va_end(vl);
-
-    logger->message( LOG_LEVEL_DEBUG, msgbuff );
+    PARSE_AND_LOG(LOG_LEVEL_DEBUG);
 }
 
 void infoLog(const char *str, ...) {
-
-    if(!logger || logger->getLevel() > LOG_LEVEL_INFO) return;
-
-    char msgbuff[65536];
-
-    va_list vl;
-
-    va_start(vl, str);
-        vsnprintf(msgbuff, 65536, str, vl);
-    va_end(vl);
-
-    logger->message( LOG_LEVEL_INFO, msgbuff );
+    PARSE_AND_LOG(LOG_LEVEL_INFO);
 }
 
 void errorLog(const char *str, ...) {
-
-    if(!logger || logger->getLevel() > LOG_LEVEL_ERROR) return;
-
-    char msgbuff[65536];
-
-    va_list vl;
-
-    va_start(vl, str);
-        vsnprintf(msgbuff, 65536, str, vl);
-    va_end(vl);
-
-    logger->message( LOG_LEVEL_ERROR, msgbuff );
+    PARSE_AND_LOG(LOG_LEVEL_ERROR)
 }
 
-Logger* logger = new Logger(LOG_LEVEL_ERROR, stderr, 0);
+void consoleLog(const char *str, ...) {
+    PARSE_AND_LOG(LOG_LEVEL_CONSOLE);
+}
+
+void scriptLog(const char *str, ...) {
+    PARSE_AND_LOG(LOG_LEVEL_SCRIPT);
+}
+
+void pedanticLog(const char *str, ...) {
+    PARSE_AND_LOG(LOG_LEVEL_PEDANTIC);
+}
+
+Logger* Logger::default_logger = new Logger(LOG_LEVEL_ERROR, stderr, 0);
