@@ -33,6 +33,11 @@ MouseCursor::MouseCursor() {
     hidden        = false;
     timeout       = 3.0f;
     idle          = timeout;
+    scrollwheel   = 0;
+    
+    left_click   = false;
+    right_click  = false;
+    middle_click = false;
     
     sdl_default_cursor = SDL_GetCursor();
     
@@ -74,6 +79,30 @@ void MouseCursor::showCursor(bool show) {
     }
 }
 
+void MouseCursor::leftClick(bool click) {
+    left_click = click;
+}
+
+void MouseCursor::rightClick(bool click) {
+    right_click = click;
+}
+
+void MouseCursor::middleClick(bool click) {
+    middle_click = click;
+}
+
+bool MouseCursor::leftClick() const {
+    return left_click;
+}
+
+bool MouseCursor::rightClick() const {
+    return right_click;    
+}
+
+bool MouseCursor::middleClick() const {
+    return middle_click;
+}
+
 bool MouseCursor::leftButtonPressed() const {
     Uint8 ms = SDL_GetMouseState(0,0);
     return ms & SDL_BUTTON(SDL_BUTTON_LEFT);
@@ -94,6 +123,10 @@ bool MouseCursor::buttonPressed() const {
     return (ms & SDL_BUTTON(SDL_BUTTON_RIGHT) || ms & SDL_BUTTON(SDL_BUTTON_LEFT));
 }
 
+int MouseCursor::scrollWheel() const {
+    return scrollwheel;    
+}
+
 bool MouseCursor::isSystemCursor() const {
     return system_cursor;
 }
@@ -123,6 +156,17 @@ void MouseCursor::updatePos(const vec2& pos) {
     idle     = 0.0f;
 }
 
+void MouseCursor::scroll(bool dir) {
+    scrollwheel += dir ? 1 : -1;
+}
+
+void MouseCursor::resetButtonState() {
+    scrollwheel  = 0;
+    right_click  = false;
+    left_click   = false;
+    middle_click = false;
+}
+
 void MouseCursor::logic(float dt) {
     idle += dt;
 }
