@@ -26,6 +26,7 @@
 */
 
 #include "sdlapp.h"
+#include "SDL_syswm.h"
 
 std::string gSDLAppResourceDir;
 std::string gSDLAppConfDir;
@@ -40,46 +41,35 @@ std::string gSDLAppTitle = "SDL App";
 std::string gSDLAppExec  = "sdlapp";
 
 #ifdef _WIN32
-HWND gSDLAppConsoleWindow = 0;
-bool using_parent_console = false;
 
-bool SDLAppAttachToConsole() {
-    if(using_parent_console)      return true;
-    if(gSDLAppConsoleWindow != 0) return false;
+HWND SDLApp::console_window   = 0;
+bool SDLApp::existing_console = false;
 
-    // if TERM is set to msys, try and attach to console
-    // could possibly add other supported TERMs here if there are any
+void SDLApp::showConsole(bool show) {
+    if(!SDLApp::console_window) return;
 
-    char* term = getenv("TERM");
-    if(!term || strcmp(term, "msys")!=0) return false;
-
-    if(AttachConsole(ATTACH_PARENT_PROCESS)) {
-
-        // send stdout to console unless already redirected
-        if (_fileno(stdout) == -1 || _get_osfhandle(fileno(stdout)) == -1) {
-            freopen("conout$","w", stdout);
-        }
-
-        // send stderr to console unless already redirected
-        if (_fileno(stderr) == -1 || _get_osfhandle(fileno(stderr)) == -1) {
-            freopen("conout$","w", stderr);
-        }
-
-        using_parent_console = true;
-    }
-
-    return using_parent_console;
+    ShowWindow( SDLApp::console_window, show);
 }
 
-void SDLAppCreateWindowsConsole() {
-    if(using_parent_console || gSDLAppConsoleWindow != 0) return;
+void SDLApp::initConsole() {
+    if(SDLApp::console_window != 0) return;
 
-    //try to attach to the available console if there is one
+    SDLApp::console_window = GetConsoleWindow();
 
-    if(SDLAppAttachToConsole()) return;
+    // check if this is a new console or not
+    CONSOLE_SCREEN_BUFFER_INFO buffer_info;
+    if(GetConsoleScreenBufferInfo( GetStdHandle( STD_OUTPUT_HANDLE ), &buffer_info )) {
+        existing_console = !(buffer_info.dwCursorPosition.X==0 && buffer_info.dwCursorPosition.Y==0);
+    }
 
-    //create a console on Windows so users can see messages
-    //find an available name for our window
+    // don't customize the console unless it was created for this program
+    if(existing_console) return;
+
+    //disable the close button so the user cant crash the application
+    HMENU hm = GetSystemMenu(SDLApp::console_window, false);
+    DeleteMenu(hm, SC_CLOSE, MF_BYCOMMAND);
+
+    //set title
 
     char consoleTitle[512];
     int console_suffix = 0;
@@ -89,33 +79,15 @@ void SDLAppCreateWindowsConsole() {
         sprintf(consoleTitle, "%s Console %d", gSDLAppTitle.c_str(), ++console_suffix);
     }
 
-    AllocConsole();
     SetConsoleTitle(consoleTitle);
-
-    //redirect streams to console
-    freopen("conin$", "r", stdin);
-    freopen("conout$","w", stdout);
-    freopen("conout$","w", stderr);
-
-    gSDLAppConsoleWindow = 0;
-
-    //wait for our console window
-    while(gSDLAppConsoleWindow==0) {
-        gSDLAppConsoleWindow = FindWindow(0, consoleTitle);
-        SDL_Delay(100);
-    }
-
-    //disable the close button so the user cant crash the application
-    HMENU hm = GetSystemMenu(gSDLAppConsoleWindow, false);
-    DeleteMenu(hm, SC_CLOSE, MF_BYCOMMAND);
 }
 
-void SDLAppResizeWindowsConsole(int height) {
-    if(gSDLAppConsoleWindow !=0) {
+void SDLApp::resizeConsole(int height) {
+    if(SDLApp::console_window !=0 && !existing_console) {
         RECT windowRect;
-        if(GetWindowRect(gSDLAppConsoleWindow, &windowRect)) {
+        if(GetWindowRect(SDLApp::console_window, &windowRect)) {
             float width = windowRect.right - windowRect.left;
-            MoveWindow(gSDLAppConsoleWindow,windowRect.left,windowRect.top,width,height,true);
+            MoveWindow(SDLApp::console_window,windowRect.left,windowRect.top,width,height,true);
         }
     }
 }
@@ -140,13 +112,13 @@ std::string SDLAppAddSlash(std::string path) {
 //info message
 void SDLAppInfo(std::string msg) {
 #ifdef _WIN32
-    SDLAppCreateWindowsConsole();
+    SDLApp::showConsole(true);
 #endif
 
     printf("%s\n", msg.c_str());
 
 #ifdef _WIN32
-    if(gSDLAppConsoleWindow) {
+    if(!SDLApp::existing_console) {
         printf("\nPress Enter\n");
         getchar();
     }
@@ -160,14 +132,14 @@ void SDLAppQuit(std::string error) {
     SDL_Quit();
 
 #ifdef _WIN32
-    SDLAppCreateWindowsConsole();
+    SDLApp::showConsole(true);
 #endif
 
     fprintf(stderr, "%s: %s\n", gSDLAppExec.c_str(), error.c_str());
     fprintf(stderr, "Try '%s --help' for more information.\n\n", gSDLAppExec.c_str());
 
 #ifdef _WIN32
-    if(gSDLAppConsoleWindow) {
+    if(!SDLApp::existing_console) {
         fprintf(stderr, "Press Enter\n");
         getchar();
     }
@@ -176,6 +148,230 @@ void SDLAppQuit(std::string error) {
     exit(1);
 }
 
+#ifdef USE_X11
+static Atom xa_targets;
+static Atom xa_clipboard;
+
+void SDLApp::initX11ClipboardEventFilter() {
+    SDL_SysWMinfo wininfo;
+    SDL_VERSION(&wininfo.version);
+    SDL_GetWMInfo(&wininfo);
+
+    SDL_EventState(SDL_SYSWMEVENT, SDL_ENABLE);
+    SDL_SetEventFilter(SDLApp::X11ClipboardEventFilter);
+
+    xa_targets   = XInternAtom(wininfo.info.x11.display, "TARGETS",   False);
+    xa_clipboard = XInternAtom(wininfo.info.x11.display, "CLIPBOARD", False);
+}
+
+int SDLApp::X11ClipboardEventFilter(const SDL_Event *event) {
+
+    if ( event->type != SDL_SYSWMEVENT ) return 1;
+
+    if( event->syswm.msg->event.xevent.type == SelectionRequest) {
+
+        SDL_SysWMinfo wininfo;
+        SDL_VERSION(&wininfo.version);
+        SDL_GetWMInfo(&wininfo);
+
+        XSelectionRequestEvent *req;
+        XEvent snotify;
+
+        req = &event->syswm.msg->event.xevent.xselectionrequest;
+
+        snotify.xselection.type      = SelectionNotify;
+
+        snotify.xselection.display   = req->display;
+        snotify.xselection.selection = req->selection;
+        snotify.xselection.requestor = req->requestor;
+        snotify.xselection.time      = req->time;
+
+        snotify.xselection.target    = None;
+        snotify.xselection.property  = None;
+
+        Atom supported_targets[] = {
+            xa_targets,
+            XA_STRING
+        };
+
+        if ( req->target == xa_targets ) {
+
+            XChangeProperty (req->display, req->requestor, req->property, XA_ATOM, 32, PropModeReplace, (unsigned char*) supported_targets, sizeof(supported_targets) / sizeof (Atom));
+            snotify.xselection.property = req->property;
+
+        } else if( req->target == XA_STRING ) {
+
+            unsigned char *selection_data;
+            unsigned long selection_length;
+            unsigned long overflow;
+            int selection_format;
+
+            if ( XGetWindowProperty(wininfo.info.x11.display, DefaultRootWindow(wininfo.info.x11.display),
+                                    XA_CUT_BUFFER0, 0, INT_MAX/4, False, req->target,
+                                    &snotify.xselection.target, &selection_format,
+                                    &selection_length, &overflow, &selection_data) == Success ) {
+
+                if ( snotify.xselection.target == req->target ) {
+
+                    XChangeProperty(wininfo.info.x11.display, req->requestor, req->property,
+                        snotify.xselection.target, selection_format, PropModeReplace, selection_data, selection_length);
+
+                    snotify.xselection.property = req->property;
+                }
+
+                XFree(selection_data);
+            }
+        }
+
+        XSendEvent(wininfo.info.x11.display,req->requestor,False,0,&snotify);
+        XSync(wininfo.info.x11.display, False);
+    }
+
+    return 1;
+}
+
+#endif
+
+bool SDLApp::getClipboardText(std::string& text) {
+
+    SDL_SysWMinfo wininfo;
+    SDL_VERSION(&wininfo.version);
+    SDL_GetWMInfo(&wininfo);
+
+#if defined(_WIN32)
+    if(!IsClipboardFormatAvailable(CF_TEXT) || !OpenClipboard(wininfo.window)) return false;
+
+    HGLOBAL handle = GetClipboardData(CF_TEXT);
+
+    if (!handle) {
+        CloseClipboard();
+        return false;
+    }
+
+    const char* global_str = (const char*) GlobalLock(handle);
+
+    text.assign(global_str);
+
+    GlobalUnlock(handle);
+
+    CloseClipboard();
+
+    return true;
+
+#elif defined(USE_X11)
+    Window owner;
+    Atom selection;
+
+    wininfo.info.x11.lock_func();
+
+    owner = XGetSelectionOwner(wininfo.info.x11.display, xa_clipboard);
+
+    wininfo.info.x11.unlock_func();
+
+    if ( (owner == None) || (owner == wininfo.info.x11.window) ) {
+
+        owner     = DefaultRootWindow(wininfo.info.x11.display);
+        selection = XA_CUT_BUFFER0;
+
+    } else {
+
+        owner = wininfo.info.x11.window;
+
+        wininfo.info.x11.lock_func();
+
+        selection = XInternAtom(wininfo.info.x11.display, "SDL_SELECTION", False);
+
+        XConvertSelection(wininfo.info.x11.display, xa_clipboard, XA_STRING, selection, owner, CurrentTime);
+
+        wininfo.info.x11.unlock_func();
+
+        int selection_response = 0;
+        SDL_Event event;
+
+        while ( !selection_response ) {
+            SDL_WaitEvent(&event);
+
+            if ( event.type == SDL_SYSWMEVENT ) {
+                XEvent xevent = event.syswm.msg->event.xevent;
+
+                if ( (xevent.type == SelectionNotify) && (xevent.xselection.requestor == owner) )
+                    selection_response = 1;
+            }
+        }
+    }
+
+    wininfo.info.x11.lock_func();
+
+    unsigned char *selection_data;
+    unsigned long selection_length;
+    unsigned long overflow;
+    int selection_format;
+    Atom selection_type;
+
+    bool assigned = false;
+
+    if ( XGetWindowProperty(wininfo.info.x11.display, owner, selection, 0, INT_MAX/4,
+                            False, XA_STRING, &selection_type, &selection_format,
+                       &selection_length, &overflow, &selection_data) == Success ) {
+
+        if ( selection_type == XA_STRING ) {
+            text.assign((const char*)selection_data);
+            assigned = true;
+        }
+
+        XFree(selection_data);
+    }
+
+    wininfo.info.x11.unlock_func();
+
+    return assigned;
+#else
+    return false;
+#endif
+}
+
+void SDLApp::setClipboardText(const std::string& text) {
+    SDL_SysWMinfo wininfo;
+    SDL_VERSION(&wininfo.version);
+    SDL_GetWMInfo(&wininfo);
+#if defined(_WIN32)
+
+    if (!OpenClipboard(wininfo.window)) return;
+
+    HANDLE handle  = GlobalAlloc((GMEM_MOVEABLE|GMEM_DDESHARE), text.size());
+
+    if(!handle) {
+        CloseClipboard();
+        return;
+    }
+
+    char* global_str = (char*) GlobalLock(handle);
+
+    strcpy(global_str, text.c_str());
+
+    GlobalUnlock(handle);
+
+    EmptyClipboard();
+
+    SetClipboardData(CF_TEXT, handle);
+
+    CloseClipboard();
+
+#elif defined(USE_X11)
+    wininfo.info.x11.lock_func();
+
+    XChangeProperty(wininfo.info.x11.display, DefaultRootWindow(wininfo.info.x11.display), XA_CUT_BUFFER0, XA_STRING, 8, PropModeReplace, (unsigned char*) text.c_str(), text.size());
+
+    if(XGetSelectionOwner(wininfo.info.x11.display, xa_clipboard) != wininfo.info.x11.window ) {
+        XSetSelectionOwner(wininfo.info.x11.display, xa_clipboard, wininfo.info.x11.window, CurrentTime);
+    }
+
+    wininfo.info.x11.unlock_func();
+#else
+
+#endif
+}
+
 void SDLAppInit(std::string apptitle, std::string execname) {
     gSDLAppTitle = apptitle;
     gSDLAppExec  = execname;
@@ -185,6 +381,7 @@ void SDLAppInit(std::string apptitle, std::string execname) {
     std::string fonts_dir    = "data/fonts/";
     std::string shaders_dir  = "data/shaders/";
 #ifdef _WIN32
+
     char szAppPath[MAX_PATH];
     GetModuleFileName(0, szAppPath, MAX_PATH);
 
@@ -242,6 +439,7 @@ void SDLAppInit(std::string apptitle, std::string execname) {
 
     gSDLAppResourceDir = resource_dir;
     gSDLAppConfDir     = conf_dir;
+    gSDLAppShaderDir   = shaders_dir;
 
     fontmanager.init();
 }
@@ -249,7 +447,7 @@ void SDLAppInit(std::string apptitle, std::string execname) {
 void SDLAppParseArgs(int argc, char *argv[], int* xres, int* yres, bool* fullscreen, std::vector<std::string>* otherargs) {
 
     for (int i=1; i<argc; i++) {
-        debugLog("argv[%d] = %s\n", i, argv[i]);
+        debugLog("argv[%d] = %s", i, argv[i]);
 
         std::string args(argv[i]);
 
@@ -281,7 +479,7 @@ void SDLAppParseArgs(int argc, char *argv[], int* xres, int* yres, bool* fullscr
                 int height = atoi(heightstr.c_str());
 
                 if(width>0 && height>0) {
-                    debugLog("w=%d, h=%d\n",width,height);
+                    debugLog("w=%d, h=%d",width,height);
 
                     *xres = width;
                     *yres = height;
@@ -329,6 +527,10 @@ void SDLApp::stop(int return_code) {
 
 int SDLApp::run() {
 
+#ifdef USE_X11
+    SDLApp::initX11ClipboardEventFilter();
+#endif
+
     Uint32 msec=0, last_msec=0, buffer_msec=0, total_msec = 0;
 
     frame_count = 0;
