@@ -51,14 +51,6 @@ extern std::string gSDLAppPathSeparator;
 extern std::string gSDLAppTitle;
 extern std::string gSDLAppExec;
 
-#ifdef _WIN32
-extern HWND gSDLAppConsoleWindow;
-
-bool SDLAppAttachToConsole();
-void SDLAppCreateWindowsConsole();
-void SDLAppResizeWindowsConsole(int height);
-#endif
-
 void SDLAppInfo(std::string msg);
 void SDLAppQuit(std::string error);
 
@@ -72,14 +64,15 @@ class SDLAppException : public std::exception {
 protected:
     std::string message;
     bool showhelp;
+
 public:
     SDLAppException(const char* str, ...) : showhelp(false) {
 
         va_list vl;
-        char msg[4096];
+        char msg[65536];
 
         va_start(vl, str);
-            vsnprintf(msg, 4096, str, vl);
+            vsnprintf(msg, 65536, str, vl);
         va_end(vl);
 
         message = std::string(msg);
@@ -105,14 +98,32 @@ protected:
     int  min_delta_msec;
     bool appFinished;
     void stop(int return_code);
+
 public:
     float fps;
 
     SDLApp();
     virtual ~SDLApp() {};
 
+#ifdef USE_X11
+    static void initX11ClipboardEventFilter();
+    static int X11ClipboardEventFilter(const SDL_Event *event);
+#endif
+
+#ifdef _WIN32
+    static HWND console_window;
+    static bool existing_console;
+
+    static void initConsole();
+    static void showConsole(bool show);
+    static void resizeConsole(int height);
+#endif
+
     int run();
 
+    static bool getClipboardText(std::string& text);
+    static void setClipboardText(const std::string& text);
+
     virtual void resize(int width, int height) {};
 
     virtual void update(float t, float dt) {};
@@ -122,7 +133,7 @@ public:
     virtual void draw(float t, float dt) {};
 
     virtual void quit() { appFinished = true; };
-    
+
     virtual void mouseMove(SDL_MouseMotionEvent *e) {};
     virtual void mouseClick(SDL_MouseButtonEvent *e) {};
     virtual void keyPress(SDL_KeyboardEvent *e) {};
