@@ -18,10 +18,10 @@
 #ifndef SDLAPP_SETTINGS_H
 #define SDLAPP_SETTINGS_H
 
-#include "regex.h"
-
 #include "conffile.h"
 
+#include <vector>
+
 class SDLAppSettings {
 protected:
     std::string default_section_name;
