@@ -31,7 +31,7 @@
 #include "SDL_image.h"
 
 #include "resource.h"
-#include "display.h"
+#include "gl.h"
 
 class TextureException : public ResourceException {
 public:
@@ -39,20 +39,59 @@ public:
 };
 
 class TextureResource : public Resource {
-    int colourFormat(SDL_Surface* surface);
-    void loadTexture(std::string file, bool mipmaps, bool clamp, bool trilinear, bool external_file);
+    bool mipmaps;
+    GLint wrap;
+    GLint min_filter;
+    GLint mag_filter;
+    std::string filename;
+
+    GLenum colourFormat(SDL_Surface* surface);
 public:
-	int w, h;
+    int w, h;
+    GLenum target;
+    GLenum format;
     GLuint textureid;
-    TextureResource(std::string name, bool mipmaps, bool clamp, bool trilinear, bool external_file);
+    GLubyte* data;
+
+    TextureResource();
+    TextureResource(int width, int height,  bool mipmaps, GLint wrap, GLenum format, GLubyte* data = 0);
+    TextureResource(const std::string& filename, bool mipmaps, GLint wrap, bool external);
+
+    void setWrapStyle(GLint wrap);
+
+    void setFiltering(GLint min_filter, GLint mag_filter);
+    void setDefaultFiltering();
+
+    void bind();
+
+    void createTexture();
+
+    void reload();
+
+    void load(bool reload = false);
+
+    void unload();
+
     ~TextureResource();
 };
 
 class TextureManager : public ResourceManager {
+    int  resource_seq;
+
+    void addResource(TextureResource* r);
 public:
+    bool trilinear;
+
     TextureManager();
-    TextureResource* grabFile(std::string name, bool mipmaps=true, bool clamp=true, bool trilinear=false);
-    TextureResource* grab(std::string file, bool mipmaps=true, bool clamp=true, bool trilinear=false, bool external_file = false);
+
+    TextureResource* grabFile(const std::string& filename, bool mipmaps = true, GLint wrap = GL_CLAMP_TO_EDGE);
+    TextureResource*     grab(const std::string& filename, bool mipmaps = true, GLint wrap = GL_CLAMP_TO_EDGE, bool external_file = false);
+
+    TextureResource* create(int width, int height, bool mipmaps, GLint wrap, GLenum format, GLubyte* data  = 0);
+    TextureResource* create(GLenum target = GL_TEXTURE_2D);
+
+    void unload();
+    void reload();
 };
 
 extern TextureManager texturemanager;
