@@ -39,16 +39,14 @@ RDirNode::RDirNode(RDirNode* parent, const std::string & abspath) {
     setParent(parent);
 
     accel = spos = prev_accel = vel = vec2(0.0f);
-    
+
     //NOTE: parent is always being set to 0 so this never gets called ...
-    
+
     //figure out starting position
     if(parent !=0) {
         vec2 parentPos = parent->getPos();
         vec2 offset;
 
-        RDirNode* parentP = parent->getParent();
-
         pos = parentPos;
     } else {
         pos = vec2(0.0f, 0.0f);
@@ -111,7 +109,7 @@ void RDirNode::nodeUpdated(bool userInitiated) {
 
     calcRadius();
     updateFilePositions();
-
+    if(visible && noDirs() && noFiles()) visible = false;
     if(parent !=0) parent->nodeUpdated(true);
 }
 
@@ -152,11 +150,11 @@ RDirNode* RDirNode::getParent() const{
 
 
 bool RDirNode::isDir(const std::string& path) const {
-      
+
     if(prefixedBy(path)) return true;
 
     if(path.find(abspath) != 0) return false;
-    
+
     for(std::list<RDirNode*>::const_iterator it = children.begin(); it != children.end(); it++) {
         if((*it)->isDir(path)) return true;
     }
@@ -171,7 +169,7 @@ void RDirNode::findDirs(const std::string& path, std::list<RDirNode*>& dirs) {
         dirs.push_back(this);
         return;
     }
- 
+
     for(std::list<RDirNode*>::const_iterator it = children.begin(); it != children.end(); it++) {
         (*it)->findDirs(path, dirs);
     }
@@ -422,7 +420,7 @@ bool RDirNode::addFile(RFile* f) {
 
     for(std::list<RFile*>::const_iterator it = files.begin(); it != files.end(); it++) {
         RFile* file = (*it);
-       
+
         if(f->path.find(file->fullpath) == 0) {
             //fprintf(stderr, "removing %s as is actually the directory of %s\n", file->fullpath.c_str(), f->fullpath.c_str());
             file->remove(true);
@@ -584,7 +582,7 @@ float RDirNode::distanceToParent() const{
 
     float posd     = glm::length(parent->getPos() - pos);
     float distance = posd - (dir_radius + parent->getParentRadius());
-    
+
     return distance;
 }
 
@@ -596,7 +594,7 @@ void RDirNode::applyForceDir(RDirNode* node) {
     float posd2       = glm::length2(dir);
     float myradius    = getRadius();
     float your_radius = node->getRadius();
-    
+
     float sumradius = (myradius + your_radius);
 
     float distance2 = posd2 - sumradius*sumradius;
@@ -655,10 +653,10 @@ void RDirNode::applyForces(QuadTree & quadtree) {
     //  * dirs should attract to sit on the radius of the parent dir ie:
     //    should attract to distance_to_parent * normal_to_parent
 
-    
-    
+
+
     accel += gGourceForceGravity * parent_dist * normalise(parent->getPos() - pos);
-    
+
     //  * dirs should be pushed along the parent_parent to parent normal by a force smaller than the parent radius force
     RDirNode* pparent = parent->getParent();
 
@@ -773,11 +771,11 @@ void RDirNode::setInitialPosition() {
     if(parentP != 0) {
         //pos += ((parent->getPos() - parentP->getPos()).normal() * 2.0 + vec2Hash(abspath)).normal();
         pos += normalise(normalise(parent->getPos() - parentP->getPos()) * 2.0f + vec2Hash(abspath));
-       
+
     }  else {
         pos += vec2Hash(abspath);
     }
-    
+
     //the spline point
     spos = pos - (parent->getPos() - pos) * 0.5f;
     position_initialized=true;
@@ -1064,9 +1062,6 @@ void RDirNode::updateBloomVBO(bloombuf& buffer, float dt) {
 void RDirNode::drawFiles(float dt) const{
 
     if(in_frustum) {
-
-        vec4 col = getColour();
-
         glPushMatrix();
             glTranslatef(pos.x, pos.y, 0.0);
 
