@@ -43,6 +43,11 @@ Gource::Gource(FrameExporter* exporter) {
     fontmedium.dropShadow(true);
     fontmedium.roundCoordinates(false);
 
+    fontcaption = fontmanager.grab("FreeSans.ttf", gGourceSettings.caption_size);
+    fontcaption.dropShadow(true);
+    fontcaption.roundCoordinates(false);
+    fontcaption.alignTop(false);
+
     font = fontmanager.grab("FreeSans.ttf", 14);
     font.dropShadow(true);
     font.roundCoordinates(true);
@@ -72,7 +77,8 @@ Gource::Gource(FrameExporter* exporter) {
 
     //load logo
     if(gGourceSettings.logo.size() > 0) {
-        logotex = texturemanager.grabFile(gGourceSettings.logo);
+        bool mipmap_logo = !(GLEW_ARB_texture_non_power_of_two || GLEW_VERSION_2_0);
+        logotex = texturemanager.grabFile(gGourceSettings.logo, mipmap_logo);
     }
 
     //load background image
@@ -82,6 +88,7 @@ Gource::Gource(FrameExporter* exporter) {
 
     stop_position_reached=false;
 
+    reloaded     = false;
     paused       = false;
     first_read   = true;
 
@@ -135,7 +142,7 @@ Gource::Gource(FrameExporter* exporter) {
     logmill = new RLogMill(logfile);
 
     shutdown = false;
-    
+
     if(exporter!=0) setFrameExporter(exporter, gGourceSettings.output_framerate);
 
     //if recording a video or in demo mode, or multiple repos, the slider is initially hidden
@@ -211,12 +218,11 @@ void Gource::unload() {
 }
 
 void Gource::reload() {
-
-    slider.resize();
+    reloaded = true;
 }
 
 void Gource::quit() {
-    shutdown = true;    
+    shutdown = true;
 }
 
 void Gource::update(float t, float dt) {
@@ -890,9 +896,19 @@ void Gource::reset() {
         delete it->second;
     }
 
-    last_percent = 0.0;
+    for(std::list<RCaption*>::iterator it = captions.begin(); it!=captions.end();it++) {
+        delete (*it);
+    }
+
+    for(std::list<RCaption*>::iterator it = active_captions.begin(); it!=active_captions.end();it++) {
+        delete (*it);
+    }
 
     files.clear();
+    captions.clear();
+    active_captions.clear();
+
+    last_percent = 0.0;
 
     file_key.clear();
 
@@ -1024,13 +1040,53 @@ void Gource::seekTo(float percent) {
     commitlog->seekTo(percent);
 }
 
+Regex caption_regex("^(?:\\xEF\\xBB\\xBF)?([0-9]+)\\|(.+)$");
+
+void Gource::loadCaptions() {
+    if(!gGourceSettings.caption_file.size()) return;
+
+    std::ifstream cf(gGourceSettings.caption_file.c_str());
+
+    if(!cf.is_open()) return;
+
+    std::string line;
+    std::vector<std::string> matches;
+
+    time_t last_timestamp = 0;
+
+    while(std::getline(cf, line)) {
+
+        ConfFile::trim(line);
+
+        if(!caption_regex.match(line, &matches)) continue;
+
+        time_t timestamp    = atol(matches[0].c_str());
+        std::string caption = RCommitLog::filter_utf8(matches[1]);
+
+        //ignore older captions
+        if(timestamp < currtime) continue;
+
+        //ignore out of order captions
+        if(timestamp < last_timestamp) continue;
+
+        last_timestamp = timestamp;
+
+        //fprintf(stderr, "%d %s\n", timestamp, matches[1].c_str());
+
+        captions.push_back(new RCaption(caption, timestamp, fontcaption));
+    }
+
+    //fprintf(stderr, "loaded %d captions\n", captions.size());
+
+}
+
 void Gource::readLog() {
     if(stop_position_reached) return;
 
     //debugLog("readLog()\n");
 
     // read commits until either we are ahead of currtime
-    while(!commitlog->isFinished() && (commitqueue.empty() || commitqueue.back().timestamp <= currtime && commitqueue.size() < commitqueue_max_size)) {
+    while(!commitlog->isFinished() && (commitqueue.empty() || (commitqueue.back().timestamp <= currtime && commitqueue.size() < commitqueue_max_size)) ) {
 
         RCommit commit;
 
@@ -1045,7 +1101,6 @@ void Gource::readLog() {
     }
 
     if(first_read && commitqueue.empty()) {
-        fprintf(stderr, "no files on first read\n");
         throw SDLAppException("no commits found");
     }
 
@@ -1058,8 +1113,8 @@ void Gource::readLog() {
 
     bool is_finished = commitlog->isFinished();
 
-    if(   gGourceSettings.stop_at_end && is_finished
-       || gGourceSettings.stop_position > 0.0 && commitlog->isSeekable() && (is_finished || last_percent >= gGourceSettings.stop_position)) {
+    if(   (gGourceSettings.stop_at_end && is_finished)
+       || (gGourceSettings.stop_position > 0.0 && commitlog->isSeekable() && (is_finished || last_percent >= gGourceSettings.stop_position)) ) {
         stop_position_reached = true;
     }
 
@@ -1103,28 +1158,27 @@ void Gource::processCommit(RCommit& commit, float t) {
                 for(std::list<RFile*>::iterator it = dir_files.begin(); it != dir_files.end(); it++) {
                     RFile* file = *it;
 
-                    addFileAction(commit.username, cf.action, file, t);
+                    addFileAction(commit.username, cf, file, t);
                 }
             }
 
-            return;
+            continue;
         }
 
         std::map<std::string, RFile*>::iterator seen_file = files.find(cf.filename);
         if(seen_file != files.end()) file = seen_file->second;
 
         if(file == 0) {
-
             file = addFile(cf);
 
             if(!file) continue;
         }
 
-        addFileAction(commit.username, cf.action, file, t);
+        addFileAction(commit.username, cf, file, t);
     }
 }
 
-void Gource::addFileAction(const std::string& username, const std::string& action, RFile* file, float t) {
+void Gource::addFileAction(const std::string& username, const RCommitFile& cf, RFile* file, float t) {
     //create user if havent yet. do it here to ensure at least one of there files
     //was added (incase we hit gGourceSettings.max_files)
 
@@ -1140,7 +1194,7 @@ void Gource::addFileAction(const std::string& username, const std::string& actio
 
         if(gGourceSettings.highlight_all_users) user->setHighlighted(true);
         else {
-        
+
             // set the highlighted flag if name matches a highlighted user
             for(std::vector<std::string>::iterator hi = gGourceSettings.highlight_users.begin(); hi != gGourceSettings.highlight_users.end(); hi++) {
                 std::string highlight = *hi;
@@ -1157,15 +1211,15 @@ void Gource::addFileAction(const std::string& username, const std::string& actio
 
     RAction* userAction = 0;
 
-    int commitNo = commit_seq++;
+    commit_seq++;
 
-    if(action == "D") {
+    if(cf.action == "D") {
         userAction = new RemoveAction(user, file, t);
     } else {
-        if(action == "A") {
+        if(cf.action == "A") {
             userAction = new CreateAction(user, file, t);
         } else {
-            userAction = new ModifyAction(user, file, t);
+            userAction = new ModifyAction(user, file, t, cf.colour);
         }
     }
 
@@ -1345,7 +1399,6 @@ void Gource::updateTime(time_t display_time) {
 
     //display date
     char datestr[256];
-    char timestr[256];
     struct tm* timeinfo = localtime ( &display_time );
 
     strftime(datestr, 256, gGourceSettings.date_format.c_str(), timeinfo);
@@ -1391,8 +1444,6 @@ void Gource::updateCamera(float dt) {
         if(track_users && (selectedFile !=0 || selectedUser !=0)) {
             Bounds2D focusbounds;
 
-            vec3 camerapos = camera.getPos();
-
             if(selectedUser !=0) focusbounds.update(selectedUser->getPos());
             if(selectedFile !=0) focusbounds.update(selectedFile->getAbsolutePos());
 
@@ -1457,7 +1508,7 @@ void Gource::logic(float t, float dt) {
         appFinished=true;
         return;
     }
-    
+
     if(message_timer>0.0f) message_timer -= dt;
     if(splash>0.0f)        splash -= dt;
 
@@ -1465,7 +1516,7 @@ void Gource::logic(float t, float dt) {
     if(commitlog == 0) {
 
         if(!logmill->isFinished()) return;
-        
+
         commitlog = logmill->getLog();
 
         std::string error = logmill->getError();
@@ -1592,6 +1643,8 @@ void Gource::logic(float t, float dt) {
     if(currtime==0 && !commitqueue.empty()) {
         currtime   = lasttime = commitqueue[0].timestamp;
         subseconds = 0.0;
+
+        loadCaptions();
     }
 
     //set current time
@@ -1636,6 +1689,92 @@ void Gource::logic(float t, float dt) {
         commitqueue.pop_front();
     }
 
+    slider.resize();
+
+    float caption_height  = fontcaption.getMaxHeight();
+    float caption_start_y = canSeek() ? slider.getBounds().min.y - 35.0f : display.height - fontmedium.getMaxHeight() - 20.0f;
+
+    if(!gGourceSettings.title.empty()) {
+        caption_start_y = glm::min( caption_start_y, display.height - 20.0f - fontmedium.getMaxHeight() );
+    }
+
+    caption_start_y = glm::floor(caption_start_y);
+
+    if(reloaded) {
+        // reposition active captions
+        float y = caption_start_y;
+
+        for(RCaption* cap : active_captions) {
+
+            int caption_offset_x = gGourceSettings.caption_offset;
+
+            // centre
+            if(caption_offset_x == 0) {
+                caption_offset_x = (display.width / 2) - (fontcaption.getWidth(cap->getCaption()) / 2);
+            } else if(caption_offset_x < 0) {
+                caption_offset_x = display.width + caption_offset_x - fontcaption.getWidth(cap->getCaption());
+            }
+
+            cap->setPos(vec2(caption_offset_x, y));
+            y -= caption_height;
+        }
+
+        reloaded = false;
+    }
+
+    while(captions.size() > 0) {
+        RCaption* caption = captions.front();
+
+        if(caption->timestamp > currtime) break;
+
+        float y = caption_start_y;
+
+        while(1) {
+
+            bool found = false;
+
+            for(RCaption* cap : active_captions) {
+
+                if(cap->getPos().y == y) {
+                    found = true;
+                    break;
+                }
+            }
+
+            if(!found) break;
+
+            y -= caption_height;
+        }
+
+        int caption_offset_x = gGourceSettings.caption_offset;
+
+        // centre
+        if(caption_offset_x == 0) {
+            caption_offset_x = (display.width / 2) - (fontcaption.getWidth(caption->getCaption()) / 2);
+        } else if(caption_offset_x < 0) {
+            caption_offset_x = display.width + caption_offset_x - fontcaption.getWidth(caption->getCaption());
+        }
+
+        caption->setPos(vec2(caption_offset_x, y));
+
+        captions.pop_front();
+        active_captions.push_back(caption);
+    }
+
+    for(std::list<RCaption*>::iterator it = active_captions.begin(); it!=active_captions.end();) {
+         RCaption* caption = *it;
+
+         caption->logic(dt);
+
+         if(caption->isFinished()) {
+             it = active_captions.erase(it);
+             delete caption;
+             continue;
+         }
+
+         it++;
+    }
+
     //reset loop counters
     gGourceUserInnerLoops = 0;
     gGourceDirNodeInnerLoops = 0;
@@ -1775,7 +1914,7 @@ void Gource::loadingScreen() {
 
     glColor4f(1.0, 1.0, 1.0, 1.0);
 
-    const char* progress;
+    const char* progress = "";
 
     switch(int(runtime*3.0f)%4) {
         case 0:
@@ -1793,7 +1932,7 @@ void Gource::loadingScreen() {
     }
 
     const char* action = !shutdown ? "Reading Log" : "Aborting";
-    
+
     int width = font.getWidth(action);
     font.setColour(vec4(1.0f));
     font.print(display.width/2 - width/2, display.height/2 - 10, "%s%s", action, progress);
@@ -1891,10 +2030,11 @@ void Gource::drawScene(float dt) {
 }
 
 void Gource::updateAndDrawEdges() {
-    if(gGourceSettings.hide_tree) return;
 
     root->calcEdges();
 
+    if(gGourceSettings.hide_tree) return;
+
     //switch to 2d
     glMatrixMode(GL_PROJECTION);
     glPushMatrix();
@@ -1919,8 +2059,9 @@ void Gource::updateAndDrawEdges() {
 
         edge_vbo.update();
 
-        shadow_shader->use();
+        shadow_shader->setSampler2D("tex", 0);
         shadow_shader->setFloat("shadow_strength", 0.5);
+        shadow_shader->use();
 
         vec2 shadow_offset = vec2(2.0, 2.0);
 
@@ -2008,17 +2149,6 @@ void Gource::setMessage(const char* str, ...) {
 
 void Gource::screenshot() {
 
-    char* screenbuff = new char[display.width * display.height * 4];
-
-    glReadPixels(0, 0, display.width, display.height,
-                 GL_RGBA, GL_UNSIGNED_BYTE, screenbuff);
-
-    const char tga_header[12] = { 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-    short width           = display.width;
-    short height          = display.height;
-    char  bitsperpixel    = 32;
-    char  imagedescriptor = 8;
-
     //get next free recording name
     char tganame[256];
     struct stat finfo;
@@ -2033,21 +2163,8 @@ void Gource::screenshot() {
     //write tga
     std::string filename(tganame);
 
-    std::ofstream tga;
-    tga.open(filename.c_str(), std::ios::out | std::ios::binary );
-
-    if(!tga.is_open()) return;
-
-    tga.write(tga_header, 12);
-    tga.write((char*)&width, sizeof(short));
-    tga.write((char*)&height, sizeof(short));
-    tga.write(&bitsperpixel, 1);
-    tga.write(&imagedescriptor, 1);
-
-    tga.write(screenbuff, display.width * display.height * 4);
-    tga.close();
-
-    delete[] screenbuff;
+    TGAWriter tga(gGourceSettings.transparent ? 4 : 3);
+    tga.screenshot(filename);
 
     setMessage("Wrote screenshot %s", tganame);
 }
@@ -2099,8 +2216,9 @@ void Gource::drawFileShadows(float dt) {
 
     if(!gGourceSettings.ffp) {
 
-        shadow_shader->use();
+        shadow_shader->setSampler2D("tex", 0);
         shadow_shader->setFloat("shadow_strength", 0.5);
+        shadow_shader->use();
 
         glBindTexture(GL_TEXTURE_2D, gGourceSettings.file_graphic->textureid);
 
@@ -2124,8 +2242,9 @@ void Gource::drawUserShadows(float dt) {
 
     if(!gGourceSettings.ffp) {
 
-        shadow_shader->use();
+        shadow_shader->setSampler2D("tex", 0);
         shadow_shader->setFloat("shadow_strength", 0.5);
+        shadow_shader->use();
 
         vec2 shadow_offset = vec2(2.0, 2.0) * gGourceSettings.user_scale;
 
@@ -2284,7 +2403,7 @@ void Gource::draw(float t, float dt) {
         fontmanager.startBuffer();
     }
 
-    font.roundCoordinates(false);    
+    font.roundCoordinates(false);
     font.setColour(vec4(gGourceSettings.dir_colour, 1.0f));
 
     root->drawNames(font);
@@ -2313,9 +2432,10 @@ void Gource::draw(float t, float dt) {
 
         text_vbo_draw_time = SDL_GetTicks();
 
-        text_shader->use();
+        text_shader->setSampler2D("tex", 0);
         text_shader->setFloat("shadow_strength", 0.7);
         text_shader->setFloat("texel_size", font_texel_size);
+        text_shader->use();
 
         fontmanager.drawBuffer();
 
@@ -2441,12 +2561,18 @@ void Gource::draw(float t, float dt) {
         fontmedium.draw(display.width/2 - date_x_offset, 20, displaydate);
     }
 
-    if(gGourceSettings.title.size()>0) {
+    if(!gGourceSettings.title.empty()) {
         fontmedium.alignTop(false);
         fontmedium.draw(10, display.height - 10, gGourceSettings.title);
         fontmedium.alignTop(true);
     }
 
+    for(std::list<RCaption*>::iterator it = active_captions.begin(); it!=active_captions.end(); it++) {
+        RCaption* caption = *it;
+
+        caption->draw();
+    }
+
     if(message_timer>0.0f) {
          fontmedium.draw(1, 3, message);
     }
