@@ -16,6 +16,12 @@
 */
 
 #include "gource_settings.h"
+#include "core/sdlapp.h"
+
+#include <boost/filesystem.hpp>
+#include <boost/algorithm/string.hpp>
+
+#include "core/utf8/utf8.h"
 
 GourceSettings gGourceSettings;
 
@@ -23,10 +29,9 @@ GourceSettings gGourceSettings;
 void GourceSettings::help(bool extended_help) {
 
 #ifdef _WIN32
-    SDLAppCreateWindowsConsole();
-
     //resize window to fit help message
-    SDLAppResizeWindowsConsole(820);
+    SDLApp::resizeConsole(820);
+    SDLApp::showConsole(true);
 #endif
 
     printf("Gource v%s\n", GOURCE_VERSION);
@@ -52,7 +57,7 @@ void GourceSettings::help(bool extended_help) {
     printf("  -s, --seconds-per-day SECONDS    Speed in seconds per day (default: 10)\n");
     printf("      --realtime                   Realtime playback speed\n");
     printf("  -c, --time-scale SCALE           Change simulation time scale (default: 1.0)\n");
-    printf("  -e, --elasticity FLOAT           Elasticity of nodes\n\n");
+    printf("  -e, --elasticity FLOAT           Elasticity of nodes (default: 0.0)\n\n");
 
     printf("  --key                            Show file extension key\n\n");
 
@@ -62,7 +67,7 @@ void GourceSettings::help(bool extended_help) {
 
     printf("  -i, --file-idle-time SECONDS     Time files remain idle (default: 60)\n\n");
 
-    printf("  --max-files NUMBER       Max number of active files (default: 1000)\n");
+    printf("  --max-files NUMBER       Max number of files or 0 for no limit\n");
     printf("  --max-file-lag SECONDS   Max time files of a commit can take to appear\n\n");
 
     printf("  --log-command VCS        Show the VCS log command (git,svn,hg,bzr,cvs2cl)\n");
@@ -125,13 +130,19 @@ if(extended_help) {
     printf("  --selection-colour       Font colour for selected users and files.\n");
     printf("  --dir-colour             Font colour for directories.\n\n");
 
-    printf("  --hash-seed SEED         Change the seed of hash function\n\n");
+    printf("  --caption-file FILE         Caption file\n");
+    printf("  --caption-size SIZE         Caption font size\n");
+    printf("  --caption-colour FFFFFF     Caption colour in hex\n");
+    printf("  --caption-duration SECONDS  Caption duration (default: 10.0)\n");
+    printf("  --caption-offset X          Caption horizontal offset\n\n");
+
+    printf("  --hash-seed SEED         Change the seed of hash function.\n\n");
 
     printf("  --path PATH\n\n");
 }
 
     printf("PATH may be a supported version control directory, a log file, a gource config\n");
-    printf("file, or '-' to read STDIN. If ommited, gource will attempt to generate a log\n");
+    printf("file, or '-' to read STDIN. If omitted, gource will attempt to generate a log\n");
     printf("from the current directory.\n\n");
 
     if(!extended_help) {
@@ -139,7 +150,7 @@ if(extended_help) {
     }
 
 #ifdef _WIN32
-    if(gSDLAppConsoleWindow) {
+    if(!SDLApp::existing_console) {
         printf("Press Enter\n");
         getchar();
     }
@@ -151,6 +162,7 @@ if(extended_help) {
 GourceSettings::GourceSettings() {
     repo_count = 0;
     file_graphic = 0;
+    log_level = LOG_LEVEL_OFF;
 
     setGourceDefaults();
 
@@ -273,12 +285,20 @@ GourceSettings::GourceSettings() {
     arg_types["highlight-colour"]   = "string";
     arg_types["selection-colour"]   = "string";
     arg_types["dir-colour"]         = "string";
-    
+
+    arg_types["caption-file"]       = "string";
+    arg_types["caption-size"]       = "int";
+    arg_types["caption-duration"]   = "float";
+    arg_types["caption-colour"]     = "string";
+    arg_types["caption-offset"]     = "int";
+
+
 }
 
 void GourceSettings::setGourceDefaults() {
 
     path = ".";
+    default_path = true;
 
     ffp = false;
 
@@ -364,9 +384,13 @@ void GourceSettings::setGourceDefaults() {
     highlight_all_users = false;
     highlight_dirs = false;
 
-    gStringHashSeed = 31;
+    caption_file     = "";
+    caption_duration = 10.0f;
+    caption_size     = 16;
+    caption_offset   = 0;
+    caption_colour   = vec3(1.0f, 1.0f, 1.0f);
 
-    log_level = LOG_LEVEL_ERROR;
+    gStringHashSeed = 31;
 
     //delete file filters
     for(std::vector<Regex*>::iterator it = file_filters.begin(); it != file_filters.end(); it++) {
@@ -452,6 +476,8 @@ void GourceSettings::commandLineOption(const std::string& name, const std::strin
             log_level = LOG_LEVEL_INFO;
         } else if(value == "error") {
             log_level = LOG_LEVEL_ERROR;
+        } else if(value == "pedantic") {
+            log_level = LOG_LEVEL_PEDANTIC;
         }
         return;
     }
@@ -644,26 +670,39 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
             user_image_dir += std::string("/");
         }
 
-        //get jpg and png images in dir
-        DIR *dp;
-        struct dirent *dirp;
-
         user_image_map.clear();
 
-        if((dp = opendir(gGourceSettings.user_image_dir.c_str())) != 0) {
+        boost::filesystem::path image_dir_path(user_image_dir);
 
-            while ((dirp = readdir(dp)) != 0) {
-                std::string dirfile = std::string(dirp->d_name);
+        if(!is_directory(image_dir_path)) {
+             conffile.entryException(entry, "specified user-image-dir is not a directory");
+        }
 
-                size_t extpos = 0;
+        std::vector<boost::filesystem::path> image_dir_files;
 
-                if(   (extpos=dirfile.rfind(".jpg"))  == std::string::npos
-                && (extpos=dirfile.rfind(".jpeg")) == std::string::npos
-                && (extpos=dirfile.rfind(".png"))  == std::string::npos) continue;
+        try {
+            copy(boost::filesystem::directory_iterator(image_dir_path), boost::filesystem::directory_iterator(), back_inserter(image_dir_files));
+        } catch(const boost::filesystem::filesystem_error& exception) {
+             conffile.entryException(entry, "error reading specified user-image-dir");
+        }
 
+        for(boost::filesystem::path& p : image_dir_files) {
 
-                std::string image_path = gGourceSettings.user_image_dir + dirfile;
-                std::string name       = dirfile.substr(0,extpos);
+            std::string dirfile;
+
+#ifdef _WIN32
+            std::wstring dirfile_16 = p.filename().wstring();
+            utf8::utf16to8(dirfile_16.begin(), dirfile_16.end(), back_inserter(dirfile));
+#else
+            dirfile = p.filename().string();
+#endif
+            std::string file_ext = extension(p);
+            boost::algorithm::to_lower(file_ext);
+
+            if(file_ext != ".jpg" && file_ext != ".jpeg" && file_ext != ".png") continue;
+
+            std::string image_path = gGourceSettings.user_image_dir + dirfile;
+            std::string name       = dirfile.substr(0,dirfile.size() - file_ext.size());
 
 #ifdef __APPLE__
                 CFMutableStringRef help = CFStringCreateMutable(kCFAllocatorDefault, 0);
@@ -677,12 +716,63 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
                 name = data;
 #endif
 
-                debugLog("%s => %s\n", name.c_str(), image_path.c_str());
+            debugLog("%s => %s", name.c_str(), image_path.c_str());
 
-                user_image_map[name] = image_path;
-            }
+            user_image_map[name] = image_path;
+        }
+    }
 
-            closedir(dp);
+    if((entry = gource_settings->getEntry("caption-file")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify caption file (filename)");
+
+        caption_file = entry->getString();
+    }
+
+    if((entry = gource_settings->getEntry("caption-duration")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify caption duration (seconds)");
+
+        caption_duration = entry->getFloat();
+
+        if(caption_duration <= 0.0f) {
+            conffile.invalidValueException(entry);
+        }
+    }
+
+    if((entry = gource_settings->getEntry("caption-size")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify caption size");
+
+        caption_size = entry->getInt();
+
+        if(caption_size<1 || caption_size>100) {
+            conffile.invalidValueException(entry);
+        }
+    }
+
+    if((entry = gource_settings->getEntry("caption-offset")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify caption offset");
+
+        caption_offset = entry->getInt();
+    }
+
+    if((entry = gource_settings->getEntry("caption-colour")) != 0) {
+
+        if(!entry->hasValue()) conffile.entryException(entry, "specify caption colour (FFFFFF)");
+
+        int r,g,b;
+
+        std::string colstring = entry->getString();
+
+        if(entry->isVec3()) {
+            caption_colour = entry->getVec3();
+        } else if(colstring.size()==6 && sscanf(colstring.c_str(), "%02x%02x%02x", &r, &g, &b) == 3) {
+            caption_colour = vec3(r,g,b);
+            caption_colour /= 255.0f;
+        } else {
+            conffile.invalidValueException(entry);
         }
     }
 
@@ -790,7 +880,7 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
             conffile.invalidValueException(entry);
         }
     }
-    
+
     if((entry = gource_settings->getEntry("selection-colour")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify selection colour (FFFFFF)");
@@ -807,8 +897,8 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
         } else {
             conffile.invalidValueException(entry);
         }
-    }    
-    
+    }
+
     if((entry = gource_settings->getEntry("dir-colour")) != 0) {
 
         if(!entry->hasValue()) conffile.entryException(entry, "specify dir colour (FFFFFF)");
@@ -898,7 +988,7 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
 
         file_idle_time = (float) atoi(file_idle_str.c_str());
 
-        if(file_idle_time<0.0f || file_idle_time == 0.0f && file_idle_str[0] != '0' ) {
+        if(file_idle_time<0.0f || (file_idle_time == 0.0f && file_idle_str[0] != '0') ) {
             conffile.invalidValueException(entry);
         }
         if(file_idle_time==0.0f) {
@@ -998,7 +1088,7 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
 
         max_files = entry->getInt();
 
-        if(max_files<0 || max_files == 0 && entry->getString() != "0") {
+        if( max_files<0 || (max_files == 0 && entry->getString() != "0") ) {
             conffile.invalidValueException(entry);
         }
     }
@@ -1163,6 +1253,7 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
     //validate path
     if(gource_settings->hasValue("path")) {
         path = gource_settings->getString("path");
+        default_path = false;
     }
 
     if(path == "-") {
@@ -1205,9 +1296,5 @@ void GourceSettings::importGourceSettings(ConfFile& conffile, ConfSection* gourc
 
         if(rc==0 && fileinfo.st_mode & S_IFDIR) isdir = true;
     }
-
-    if(path.size()==0 || isdir) {
-        SDLAppCreateWindowsConsole();
-    }
 #endif
 }
