@@ -26,9 +26,10 @@ class GourceShell : public SDLApp {
 
     Gource* gource;
 
+    bool shutdown;
     bool next;
 
-    GLuint transition_texture;
+    TextureResource* transition_texture;
     float  transition_interval; 
 
     FrameExporter* exporter;
@@ -43,10 +44,20 @@ public:
 
     void update(float t, float dt);
 
+    void resize(int width, int height);
+
+    void toggleFullscreen();
+    
+    void quit();
+        
     void keyPress(SDL_KeyboardEvent *e);
     void mouseMove(SDL_MouseMotionEvent *e);
     void mouseClick(SDL_MouseButtonEvent *e);
-
+#if SDL_VERSION_ATLEAST(1,3,0)
+    void mouseWheel(SDL_MouseWheelEvent *e);
+#endif
 };
 
 #endif
+
+extern GourceShell* gGourceShell;
