@@ -21,6 +21,10 @@ int main(int argc, char *argv[]) {
 
     SDLAppInit("Gource", "gource");
 
+#ifdef _WIN32
+        SDLApp::initConsole();
+#endif
+
     ConfFile conf;
     std::vector<std::string> files;
 
@@ -38,9 +42,9 @@ int main(int argc, char *argv[]) {
 
                 int file_length = file.size();
 
-                if(   file.rfind(".conf") == (file_length-5) && file_length > 5
-                   || file.rfind(".cfg")  == (file_length-4) && file_length > 4
-                   || file.rfind(".ini")  == (file_length-4) && file_length > 4) {
+                if(   (file.rfind(".conf") == (file_length-5) && file_length > 5)
+                   || (file.rfind(".cfg")  == (file_length-4) && file_length > 4)
+                   || (file.rfind(".ini")  == (file_length-4) && file_length > 4) ) {
 
                     bool is_conf=true;
 
@@ -61,7 +65,14 @@ int main(int argc, char *argv[]) {
         }
 
         //set log level
-        logger->setLevel(gGourceSettings.log_level);
+        Logger::getDefault()->setLevel(gGourceSettings.log_level);
+
+#ifdef _WIN32
+        // hide console if not needed
+        if(gGourceSettings.log_level == LOG_LEVEL_OFF && !SDLApp::existing_console) {
+            SDLApp::showConsole(false);
+        }
+#endif
 
         //load config
         if(!gGourceSettings.load_config.empty()) {
@@ -126,7 +137,7 @@ int main(int argc, char *argv[]) {
     if(gGourceSettings.resizable && gGourceSettings.output_ppm_filename.empty()) {
         display.enableResize(true);
     }
-        
+
     try {
 
         display.init("Gource", gGourceSettings.display_width, gGourceSettings.display_height, gGourceSettings.fullscreen);
@@ -139,10 +150,6 @@ int main(int argc, char *argv[]) {
         SDLAppQuit(errormsg);
     }
 
-#ifdef _WIN32
-    SDLAppAttachToConsole();
-#endif
-
     //init frame exporter
     FrameExporter* exporter = 0;
 
@@ -186,7 +193,7 @@ int main(int argc, char *argv[]) {
             SDLAppQuit(exception.what());
         }
     }
-    
+
     gGourceShell = 0;
 
     if(gourcesh != 0) delete gourcesh;
